package ru.dbotthepony.mc.otm.worldgen.placement

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.util.RandomSource
import net.minecraft.world.level.levelgen.placement.PlacementContext
import net.minecraft.world.level.levelgen.placement.PlacementModifier
import net.minecraft.world.level.levelgen.placement.PlacementModifierType
import ru.dbotthepony.mc.otm.registry.data.MPlacementModifiers
import ru.dbotthepony.mc.otm.worldgen.EnhancedPlacementContext
import java.util.stream.Collectors
import java.util.stream.Stream

/**
 * Or "shard" placement, if you will.
 *
 * Basically, allows multiple [PlacementModifier]s to split/branch off from provided point.
 */
class EnhancedSplitPlacement(
	val children: List<EnhancedPlacementModifier>
) : EnhancedPlacementModifier {
	constructor(vararg children: EnhancedPlacementModifier) : this(ImmutableList.copyOf(children))

	override fun evaluate(context: EnhancedPlacementContext, positions: List<BlockPos>): List<BlockPos> {
		val result = ArrayList<BlockPos>()
		children.forEach { result.addAll(it.evaluate(context, positions)) }
		return result
	}

	override val type: EnhancedPlacementModifier.Type<*>
		get() = Companion

	companion object : EnhancedPlacementModifier.Type<EnhancedSplitPlacement> {
		override val codec: MapCodec<EnhancedSplitPlacement> by lazy {
			Codec.list(EnhancedPlacementModifier.CODEC).xmap(::EnhancedSplitPlacement, EnhancedSplitPlacement::children).fieldOf("children")
		}
	}
}
